//---------------------------------
// 2019/3/4

#include <Wire.h>
#include "CCS811.h"


//---------------------------------
// Initialization

bool CCS811::begin(uint8_t addrI2C){
  Wire.begin();
  this->addrI2C = addrI2C;
  if(!readID()){
    return false;
  }

  // Boot mode
  if((read(0x0)&0x80)==0){
    appStart();   // Go to application mode
  }
  return true;
}

//---------------------------------
// I2C Transmission

// Read 1Byte from register
uint8_t CCS811::read(uint8_t addr){
  Wire.beginTransmission(addrI2C);
  Wire.write(addr);
  Wire.endTransmission();
  Wire.requestFrom(addrI2C, (uint8_t)1);
  return Wire.read();
}

// Read length Byte from address
void CCS811::read(uint8_t addr, uint8_t*data, uint8_t length){
  Wire.beginTransmission(addrI2C);
  Wire.write(addr);
  Wire.endTransmission();
  Wire.requestFrom(addrI2C,length);
  for(uint8_t i=0; i<length; i++){
    data[i] = Wire.read();
  }
}

// Write 1Byte at specific address
void CCS811::write(uint8_t addr, uint8_t data){
  Wire.beginTransmission(addrI2C);
  Wire.write(addr);
  Wire.write(data);
  Wire.endTransmission();
  return;
}

// Write length Byte at specific address
void CCS811::write(uint8_t addr, uint8_t*data, uint8_t length){
  Wire.beginTransmission(addrI2C);
  Wire.write(addr);
  for(uint8_t i=0; i<length; i++){
      Wire.write(data[i]);
  }
  Wire.endTransmission();
  return;
}

// Application mode start command
void CCS811::appStart(){
  Wire.beginTransmission(addrI2C);
  Wire.write(0xF4);
  Wire.endTransmission();
  return;
}


//---------------------------------
// Register Operation

// Software Reset Command
void CCS811::reset(){
  uint8_t data[4];
  data[0] = 0x11;
  data[1] = 0xE5;
  data[2] = 0x72;
  data[3] = 0x8A;
  write(0xFF, data, 4);
}

// Read ID
//  Return true if success
bool CCS811::readID(){
  uint8_t data = read(0x20);
  if(data!=0x81){
    return false;
  }
  return true;
}


// Get drive mode
uint8_t CCS811::getDriveMode(){
  uint8_t data = read(0x1) & 0x7C;
  return data>>4;
}

// Set drive mode
void CCS811::setDriveMode(uint8_t mode){
  uint8_t data = read(0x1) & 0x0C;
  write(0x1, (mode<<4)+data);
}

// Read ALG result register and sotre result to TVOC and eCO2 variable
//  Return true if update
bool CCS811::getALGResult(){
  uint8_t status = read(0x0);
  uint8_t data[4];

  // Boot mode
  if((status&0x80)==0){
    return false;
  }

  // Error occured
  if((status&0x1)!=0){
    return false;
  }

  // Result not ready
  if((status&0x8)==0){
    return false;
  }

  read(0x2, data, 4);


  eCO2 = data[1] + (data[0]<<8);
  TVOC = data[3] + (data[2]<<8);
  return true;
}


// Return HW version, FW boot version and FW app version
String CCS811::getVersionStr(){
  String str = "";
  uint8_t data[2];
  read(0x21, data, 1);
  str = str + F("CCS811\n HW Version : 0x") + String(data[0]);
  read(0x23, data, 2);
  str = str + F("\n FW Boot Version : 0x") + String(((uint16_t)data[1]<<8) + data[0]);
  read(0x24, data, 2);
  str = str + F("\n FW App Version : 0x") + String(((uint16_t)data[1]<<8) + data[0]) + F("\n");
  return str;
}
